D			[0-9]
L			[a-zA-Z_]
H			[a-fA-F0-9]
E			[Ee][+-]?{D}+
FS			(f|F|l|L)
IS			(u|U|l|L)*
%x STATE_COMMENT    
%x STATE_STRING 
%{
#include <stdio.h>
/*#include "basava.tab.h" */
#include "basava.h"
extern void yyerror(const char *);   /* prints grammar violation message */
extern int yylineno;     // Line no for debugging messages
extern char* yytext;
extern unsigned int symbolTableAdd(const char* word);

char buf[1000];
char *s;
int column = 0;
int sym_index = 0;
void count();


%}
%%
"/*"						{count(); BEGIN(STATE_COMMENT);}
<STATE_COMMENT>.|\n			{count(); ; }
<STATE_COMMENT>"*/"			{count(); BEGIN(INITIAL);}
\"							{count(); BEGIN(STATE_STRING); s=buf; }
<STATE_STRING>\\n     { count(); *s++ = '\n'; }
<STATE_STRING>\\t     { count(); *s++ = '\t'; }
<STATE_STRING>\\\"    { count(); *s++ = '\"'; }
<STATE_STRING>\"      { count(); 
                  *s = 0;
                 /* BEGIN 0; */
                  BEGIN(INITIAL);
                  printf("found '%s'\n", buf);
                  return STRING_LITERAL;
                }
<STATE_STRING>\n      { count(); /* printf("invalid string"); exit(1); */ }
<STATE_STRING>.       { count(); *s++ = *yytext; }

"program"				   	{count(); return PROGRAM;}
"char"						{count(); return CHAR;}
"int"						{count(); return INT;}
"string"					{count(); return STRING;}
"float"						{count(); return FLOAT;}
"void"						{count(); return VOID;}
"main"						{count(); return MAIN;}
"goto"						{count(); return GOTO;}
"if"						{count(); return IF;}
"else"						{count(); return ELSE;}
"do"						{count(); return DO;}
"while"						{count(); return WHILE;}

{L}({L}|{D})*               { count(); 
								/*yylval.id = yytext[0]; */ 
								 sym_index = symbolTableAdd(yytext);
								return IDENTIFIER;}
[0]|(["+"|"-"]?[0-9]+)      		{ count(); 
								/*yylval.num = atoi(yytext); */
								return LONG_LITERAL;}
([0]|(["+"|"-"]?[0-9]+))"."[0-9]+      		{ count(); 
								/*yylval.num = atoi(yytext); */
								return DOUBLE_LITERAL;}
['].?[']					{ count(); return CHAR_LITERAL;
/* L?\"(\\.|[^\\"])*\"      	{ count(); return STRING_LITERAL;}  */
}
"..."			{ count(); return(ELLIPSIS); }
">>="			{ count(); return(RIGHT_ASSIGN); }
"<<="			{ count(); return(LEFT_ASSIGN); }
"+="			{ count(); return(ADD_ASSIGN); }
"-="			{ count(); return(SUB_ASSIGN); }
"*="			{ count(); return(MUL_ASSIGN); }
"/="			{ count(); return(DIV_ASSIGN); }
"%="			{ count(); return(MOD_ASSIGN); }
"&="			{ count(); return(AND_ASSIGN); }
"^="			{ count(); return(XOR_ASSIGN); }
"|="			{ count(); return(OR_ASSIGN); }
">>"			{ count(); return(RIGHT_OP); }
"<<"			{ count(); return(LEFT_OP); }
"++"			{ count(); return(INC_OP); }
"--"			{ count(); return(DEC_OP); }
"->"			{ count(); return(PTR_OP); }
"&&"			{ count(); return(AND_OP); }
"||"			{ count(); return(OR_OP); }
"<="			{ count(); return(LE_OP); }
">="			{ count(); return(GE_OP); }
"=="			{ count(); return(EQ_OP); }
"!="			{ count(); return(NE_OP); }
";"			{ count(); return(';'); }
("{"|"<%")		{ count(); return('{'); }
("}"|"%>")		{ count(); return('}'); }
","			{ count(); return(','); }
":"			{ count(); return(':'); }
"="			{ count(); return('='); }
"("			{ count(); return('('); }
")"			{ count(); return(')'); }
("["|"<:")		{ count(); return('['); }
("]"|":>")		{ count(); return(']'); }
"."			{ count(); return('.'); }
"&"			{ count(); return('&'); }
"!"			{ count(); return('!'); }
"~"			{ count(); return('~'); }
"-"			{ count(); return('-'); }
"+"			{ count(); return('+'); }
"*"			{ count(); return('*'); }
"/"			{ count(); return('/'); }
"%"			{ count(); return('%'); }
"<"			{ count(); return('<'); }
">"			{ count(); return('>'); }
"^"			{ count(); return('^'); }
"|"			{ count(); return('|'); }
"?"			{ count(); return('?'); }
"//"[^\n]*\n				{ count(); ;    /* Ignore single line comments  starting with double-/ */ }
[ \t\n\f\v]            		{ count(); ; }
.                      		{count(); 
								/* ignore bad characters */
								/* yyerror ("unexpected character"); */
								/* printf("Unexpected Caharacter, "); */ }

%%
int yywrap (void) {return 1;}

void count()
{
	int i;

	for (i = 0; yytext[i] != '\0'; i++)
		if (yytext[i] == '\n') {
			column = 0; yylineno++ ;
		}
		else if (yytext[i] == '\t')
			column += 8 - (column % 8);
		else
			column++;

	// ECHO;
}
